/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
 * Description: MDC VIZ库 解析json文件
 * Create: 2020-09-10
 */
#include <fstream>
#include <unistd.h>
#include <sys/param.h>
#include <ara/log/logging.h>

#include "json_parser.h"

namespace {
ara::log::Logger &g_logger = CreateLogger("Json", "demo", ara::log::LogLevel::kVerbose);
}

void JsonParser::ParseJson(bool &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isBool()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [bool] type.");
        return;
    }
    elem = jsonValue.asBool();
}

void JsonParser::ParseJson(ara::core::String &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isString()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [string] type.");
        return;
    }
    elem = static_cast<ara::core::String>(jsonValue.asString());
}

void JsonParser::ParseJson(int32_t &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [int32] type.");
        return;
    }
    elem = static_cast<int32_t>(jsonValue.asInt());
}

void JsonParser::ParseJson(uint32_t &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isUInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [uint32] type.");
        return;
    }
    elem = static_cast<uint32_t>(jsonValue.asUInt());
}

void JsonParser::ParseJson(uint8_t &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isUInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [uint8] type.");
        return;
    }
    elem = static_cast<uint8_t>(jsonValue.asUInt());
}

void JsonParser::ParseJson(uint16_t &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isUInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [uint16] type.");
        return;
    }
    elem = static_cast<uint16_t>(jsonValue.asUInt());
}

void JsonParser::ParseJson(float &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isDouble()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [float] type.");
        return;
    }
    elem = static_cast<float>(jsonValue.asDouble());
}

void JsonParser::ParseJson(double &elem, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isDouble()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [double] type.");
        return;
    }
    elem = jsonValue.asDouble();
}

void JsonParser::ParseJson(mdc::visual::MarkerAction &action, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isUInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [MarkerAction] type.");
        return;
    }
    action = static_cast<mdc::visual::MarkerAction>(jsonValue.asUInt());
}

void JsonParser::ParseJson(mdc::visual::MarkerType &markerType, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    if (!jsonValue.isUInt()) {
        g_logger.LogError() << ("Failed to convert the value of the " + key + " key to the [MarkerType] type.");
        return;
    }
    markerType = static_cast<mdc::visual::MarkerType>(jsonValue.asUInt());
}

void JsonParser::ParseJson(mdc::visual::ColorRGBA &color, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(color.r, jsonValue, "r");
    ParseJson(color.g, jsonValue, "g");
    ParseJson(color.b, jsonValue, "b");
    ParseJson(color.a, jsonValue, "a");
}

void JsonParser::ParseJson(mdc::visual::Times &times, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(times.nsec, jsonValue, "nsecs");
    ParseJson(times.sec, jsonValue, "secs");
}

void JsonParser::ParseJson(ara::core::Vector<mdc::visual::ColorRGBA> &colors, const Json::Value &currentNode,
    const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    for (Json::ArrayIndex i = 0; i != jsonValue.size(); ++i) {
        mdc::visual::ColorRGBA color;
        ParseJson(color.r, jsonValue[i], "r");
        ParseJson(color.g, jsonValue[i], "g");
        ParseJson(color.b, jsonValue[i], "b");
        ParseJson(color.a, jsonValue[i], "a");
        colors.push_back(color);
    }
}

void JsonParser::ParseJson(mdc::visual::Header &header, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(header.frameId, jsonValue, "frame_id");
    ParseJson(header.seq, jsonValue, "seq");
    ParseJson(header.stamp, jsonValue, "stamp");
}

void JsonParser::ParseJson(mdc::visual::Point &point, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(point.x, jsonValue, "x");
    ParseJson(point.y, jsonValue, "y");
    ParseJson(point.z, jsonValue, "z");
}

void JsonParser::ParseJson(ara::core::Vector<mdc::visual::Point> &points, const Json::Value &currentNode,
    const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    for (Json::ArrayIndex i = 0; i != jsonValue.size(); ++i) {
        mdc::visual::Point point;
        ParseJson(point.x, jsonValue[i], "x");
        ParseJson(point.y, jsonValue[i], "y");
        ParseJson(point.z, jsonValue[i], "z");
        points.push_back(point);
    }
}

void JsonParser::ParseJson(mdc::visual::Quaternion &quaternion, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(quaternion.x, jsonValue, "x");
    ParseJson(quaternion.y, jsonValue, "y");
    ParseJson(quaternion.z, jsonValue, "z");
    ParseJson(quaternion.w, jsonValue, "w");
}

void JsonParser::ParseJson(mdc::visual::Pose &pose, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(pose.orientation, jsonValue, "orientation");
    ParseJson(pose.position, jsonValue, "position");
}

void JsonParser::ParseJson(mdc::visual::Vector3 &vec, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(vec.x, jsonValue, "x");
    ParseJson(vec.y, jsonValue, "y");
    ParseJson(vec.z, jsonValue, "z");
}

void JsonParser::ParseJson(mdc::visual::Marker &marker, const Json::Value &currentNode)
{
    ParseJson(marker.action, currentNode, "action");
    ParseJson(marker.color, currentNode, "color");
    ParseJson(marker.colors, currentNode, "colors");
    ParseJson(marker.frameLocked, currentNode, "frame_locked");
    ParseJson(marker.header, currentNode, "header");
    ParseJson(marker.id, currentNode, "id");
    ParseJson(marker.lifetime, currentNode, "lifeTime");
    ParseJson(marker.meshResource, currentNode, "mesh_resource");
    ParseJson(marker.meshUseEmbeddedMaterials, currentNode, "mesh_use_embedded_materials");
    ParseJson(marker.ns, currentNode, "ns");
    ParseJson(marker.points, currentNode, "points");
    ParseJson(marker.pose, currentNode, "pose");
    ParseJson(marker.scale, currentNode, "scale");
    ParseJson(marker.type, currentNode, "type");
}

void JsonParser::ParseJson(mdc::visual::MarkerArray &markerArray, const Json::Value &root)
{
    Json::Value currentNode;
    for (Json::ArrayIndex i = 0; i != root.size(); ++i) {
        mdc::visual::Marker marker;
        currentNode = root[i];
        ParseJson(marker, currentNode);
        markerArray.markers.push_back(marker);
    }
}

void JsonParser::ParseJson(mdc::visual::RadarDetectArray &detectArray, const Json::Value &currentNode)
{
    ParseJson(detectArray.header, currentNode, "header");
    ParseJson(detectArray.sensorId, currentNode, "sensorId");
    ParseJson(detectArray.radarState, currentNode, "radarState");
    ParseJson(detectArray.detectList, currentNode, "detectList");
}

void JsonParser::ParseJson(mdc::visual::RadarTrackArray &trackArray, const Json::Value &currentNode)
{
    ParseJson(trackArray.header, currentNode, "header");
    ParseJson(trackArray.sensorId, currentNode, "sensorId");
    ParseJson(trackArray.radarState, currentNode, "radarState");
    ParseJson(trackArray.trackList, currentNode, "trackList");
}

void JsonParser::ParseJson(mdc::visual::RadarState &state, const Json::Value &currentNode, const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    ParseJson(state.sensorId, currentNode, "sensorId");
    ParseJson(state.nvmReadStatus, currentNode, "nvmReadStatus");
    ParseJson(state.nvmWriteStatus, currentNode, "nvmWriteStatus");
    ParseJson(state.persistentError, currentNode, "persistentError");
    ParseJson(state.temporaryError, currentNode, "temporaryError");
    ParseJson(state.interferenceError, currentNode, "interferenceError");
    ParseJson(state.temperatureError, currentNode, "temperatureError");
    ParseJson(state.voltageError, currentNode, "voltageError");
    ParseJson(state.maxDistance, currentNode, "maxDistance");
    ParseJson(state.sortIndex, currentNode, "sortIndex");
    ParseJson(state.radarPower, currentNode, "radarPower");
    ParseJson(state.ctrlRelay, currentNode, "ctrlRelay");
    ParseJson(state.outputType, currentNode, "outputType");
    ParseJson(state.sendQuality, currentNode, "sendQuality");
    ParseJson(state.sendExtinfo, currentNode, "sendExtinfo");
    ParseJson(state.motionRxState, currentNode, "motionRxState");
    ParseJson(state.rcsThreshold, currentNode, "rcsThreshold");
    ParseJson(state.blockError, currentNode, "blockError");
    ParseJson(state.broadcastError, currentNode, "broadcastError");
    ParseJson(state.electricAxisError, currentNode, "electricAxisError");
    ParseJson(state.configError, currentNode, "configError");
    ParseJson(state.calibrationError, currentNode, "calibrationError");
    ParseJson(state.connectorDirection, currentNode, "connectorDirection");
    ParseJson(state.can0WorkMode, currentNode, "can0WorkMode");
    ParseJson(state.can1WorkMode, currentNode, "can1WorkMode");
    ParseJson(state.dualCanMode, currentNode, "dualCanMode");
    ParseJson(state.timmingMode, currentNode, "timmingMode");
    ParseJson(state.powerMode, currentNode, "powerMode");
    ParseJson(state.performanceMode, currentNode, "performanceMode");
    ParseJson(state.radarPosition, currentNode, "radarPosition");
    ParseJson(state.hwError, currentNode, "hwError");
}

void JsonParser::ParseJson(mdc::visual::RadarDetect &detect, const Json::Value &currentNode)
{
    ParseJson(detect.id, currentNode, "id");
    ParseJson(detect.idPair, currentNode, "idPair");
    ParseJson(detect.coordinate, currentNode, "coordinate");
    ParseJson(detect.x, currentNode, "x");
    ParseJson(detect.y, currentNode, "y");
    ParseJson(detect.z, currentNode, "z");
    ParseJson(detect.vx, currentNode, "vx");
    ParseJson(detect.vy, currentNode, "vy");
    ParseJson(detect.rcs, currentNode, "rcs");
    ParseJson(detect.snr, currentNode, "snr");
    ParseJson(detect.xRms, currentNode, "xRms");
    ParseJson(detect.yRms, currentNode, "yRms");
    ParseJson(detect.zRms, currentNode, "zRms");
    ParseJson(detect.vxRms, currentNode, "vxRms");
    ParseJson(detect.vyRms, currentNode, "vyRms");
    ParseJson(detect.xQuality, currentNode, "xQuality");
    ParseJson(detect.yQuality, currentNode, "yQuality");
    ParseJson(detect.zQuality, currentNode, "zQuality");
    ParseJson(detect.vxQuality, currentNode, "vxQuality");
    ParseJson(detect.vyQuality, currentNode, "vyQuality");
    ParseJson(detect.existProbability, currentNode, "existProbability");
    ParseJson(detect.falseProbability, currentNode, "falseProbability");
    ParseJson(detect.movProperty, currentNode, "movProperty");
    ParseJson(detect.invalidState, currentNode, "invalidState");
    ParseJson(detect.ambiguity, currentNode, "ambiguity");
}

void JsonParser::ParseJson(ara::core::Vector<mdc::visual::RadarDetect> &detects, const Json::Value &currentNode,
    const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    for (Json::ArrayIndex i = 0; i != jsonValue.size(); ++i) {
        mdc::visual::RadarDetect detect;
        ParseJson(detect, jsonValue[i]);
        detects.push_back(detect);
    }
}

void JsonParser::ParseJson(mdc::visual::RadarTrack &track, const Json::Value &currentNode)
{
    ParseJson(track.id, currentNode, "id");
    ParseJson(track.idState, currentNode, "idState");
    ParseJson(track.lifetime, currentNode, "lifetime");
    ParseJson(track.x, currentNode, "x");
    ParseJson(track.y, currentNode, "y");
    ParseJson(track.z, currentNode, "z");
    ParseJson(track.vx, currentNode, "vx");
    ParseJson(track.vy, currentNode, "vy");
    ParseJson(track.ax, currentNode, "ax");
    ParseJson(track.ay, currentNode, "ay");
    ParseJson(track.rcs, currentNode, "rcs");
    ParseJson(track.snr, currentNode, "snr");
    ParseJson(track.xRms, currentNode, "xRms");
    ParseJson(track.yRms, currentNode, "yRms");
    ParseJson(track.zRms, currentNode, "zRms");
    ParseJson(track.vxRms, currentNode, "vxRms");
    ParseJson(track.vyRms, currentNode, "vyRms");
    ParseJson(track.axRms, currentNode, "axRms");
    ParseJson(track.ayRms, currentNode, "ayRms");
    ParseJson(track.orientation, currentNode, "orientation");
    ParseJson(track.orientRms, currentNode, "orientRms");
    ParseJson(track.yawRate, currentNode, "yawRate");
    ParseJson(track.length, currentNode, "length");
    ParseJson(track.width, currentNode, "width");
    ParseJson(track.height, currentNode, "height");
    ParseJson(track.yawRateRms, currentNode, "yawRateRms");
    ParseJson(track.lengthRms, currentNode, "lengthRms");
    ParseJson(track.widthRms, currentNode, "widthRms");
    ParseJson(track.heightRms, currentNode, "heightRms");
    ParseJson(track.xQuality, currentNode, "xQuality");
    ParseJson(track.yQuality, currentNode, "yQuality");
    ParseJson(track.zQuality, currentNode, "zQuality");
    ParseJson(track.vxQuality, currentNode, "vxQuality");
    ParseJson(track.vyQuality, currentNode, "vyQuality");
    ParseJson(track.axQuality, currentNode, "axQuality");
    ParseJson(track.ayQuality, currentNode, "ayQuality");
    ParseJson(track.orientationQuality, currentNode, "orientationQuality");
    ParseJson(track.yawRateQuality, currentNode, "yawRateQuality");
    ParseJson(track.lengthQuality, currentNode, "lengthQuality");
    ParseJson(track.widthQuality, currentNode, "widthQuality");
    ParseJson(track.heightQuality, currentNode, "heightQuality");
    ParseJson(track.underpassProbability, currentNode, "underpassProbability");
    ParseJson(track.overpassProbability, currentNode, "overpassProbability");
    ParseJson(track.existProbability, currentNode, "existProbability");
    ParseJson(track.movProperty, currentNode, "movProperty");
    ParseJson(track.trackState, currentNode, "trackState");
    ParseJson(track.trackType, currentNode, "trackType");
}

void JsonParser::ParseJson(ara::core::Vector<mdc::visual::RadarTrack> &tracks, const Json::Value &currentNode,
    const std::string &key)
{
    Json::Value defaultValue;
    const auto &jsonValue = currentNode.get(key, defaultValue);
    if (jsonValue == defaultValue) {
        return;
    }
    for (Json::ArrayIndex i = 0; i != jsonValue.size(); ++i) {
        mdc::visual::RadarTrack track;
        ParseJson(track, jsonValue[i]);
        tracks.push_back(track);
    }
}
